# -*- coding: utf-8 -*-
import os

from fabric.api import cd
from fabric import colors
from fabric.contrib.console import confirm


class VirtualenvMixin(object):
    """
    A mixin that will check for a virtual environment at <project_path>/venv
    before deployment.
    """

    def run_server_updates(self, instance):
        code_dir = self.project_paths[instance]
        self.check_virtualenv(code_dir)
        super(VirtualenvMixin, self).run_server_updates(instance)

    def check_virtualenv(self, path):
        venv_path = os.path.join(path, 'venv')
        if not self.exists(venv_path):
            if confirm('virtualenv not found. Do you want to create one'):
                print(colors.yellow('Creating virtualenv'))
                with cd(path):
                    self.run('virtualenv venv')



from fabric import colors


class Response(object):
    failed = False


class TestMixin(object):
    """
    Overrides run to only log to a file
    """

    def run(self, command):
        self.notify(colors.cyan('run(%s)' % command))
        return 'run output'

    def local(self, command, *args, **kwargs):
        self.notify(colors.cyan('local(%s)' % command))
        return Response()

    def exists(self, *args, **kwargs):
        self.notify(colors.cyan('exists(%s, %s)' % (args, kwargs)))

    def get(self, *args, **kwargs):
        self.notify(colors.cyan('get(%s, %s)' % (args, kwargs)))



# -*- coding: utf-8 -*-
import time
import django

from fabric.context_managers import settings, quiet
from fabric.contrib.console import confirm
from fabric.contrib.files import exists
from fabric.api import local, run, cd
from fabric.operations import get
from fabric.contrib import django as fab_django
from fabric.utils import abort
from fabric import colors

import requests

STATIC_FILES = 'django.contrib.staticfiles'


class App(object):
    project_paths = {}
    project_package = None
    test_settings = None
    strict = False
    restart_command = None
    loaddata_command = 'loaddata'
    dumpdata_command = 'dumpdata'
    urls = None
    local_tables_to_flush = []
    requirements = {
        'dev': 'requirements.txt',
        'prod': 'requirements.txt',
    }
    status_code = None
    virtualenv_activate = 'source venv/bin/activate'

    def __init__(self, project_paths=None, project_package=None, test_settings=None, strict=False,
                 restart_command=None, loaddata_command=None, dumpdata_command=None,
                 requirements=None, local_tables_to_flush=[], urls=None, virtualenv_activate=None):
        self.project_paths = project_paths or self.project_paths
        self.project_package = project_package or self.project_package
        self.test_settings = test_settings or self.test_settings
        self.restart_command = restart_command or self.restart_command
        self.loaddata_command = loaddata_command or self.loaddata_command
        self.dumpdata_command = dumpdata_command or self.dumpdata_command
        self.local_tables_to_flush = local_tables_to_flush or self.local_tables_to_flush
        self.requirements = requirements or self.requirements
        self.strict = strict or self.strict
        self.urls = urls or self.urls
        self.virtualenv_activate = virtualenv_activate or self.virtualenv_activate
        fab_django.project(self.project_package)

    def notify(self, message):
        print(message)

    def run(self, command):
        with quiet():
            return run(command)

    def local(self, command, *args, **kwargs):
        with quiet():
            return local(command, *args, **kwargs)

    def exists(self, *args, **kwargs):
        return exists(*args, **kwargs)

    def get(self, *args, **kwargs):
        return get(*args, **kwargs)

    def local_management_command(self, command, *args, **kwargs):
        return self.local('python manage.py %s' % command, *args, **kwargs)

    def run_management_command(self, instance, command):
        if instance == 'local':
            return self.local_management_command(command)

        code_dir = self.project_paths[instance]
        with cd(code_dir):
            return self.run('%s && python manage.py %s' % (self.virtualenv_activate, command))

    def get_head_hash(self):
        return self.run('git rev-parse HEAD')

    def test(self, is_deploying=True):
        with settings(warn_only=True):
            self.notify(colors.yellow('Running tests, please wait!'))
            if settings is None:
                command = 'test --settings=%s' % \
                          self.test_settings
            else:
                command = 'test'
            result = self.local_management_command(command, capture=True)

        if result.failed:
            self.notify(colors.red('Tests failed'))
            if is_deploying:
                if self.strict:
                    abort(colors.red('You are not allowed to deploy with '
                                     'broken tests'))
                if not confirm('Do you really want to deploy?'):
                    abort('Ok!')
        else:
            self.notify(colors.green('All tests ok'))

    def lock_value(self):
        return self.local('git config user.name', capture=True)

    def lock(self, instance):
        with cd(self.project_paths[instance]):
            if self.exists('.deploying'):
                locker = self.run('cat .deploying')
                if self.lock_value() != '' and locker == self.lock_value():
                    if not confirm(colors.red('Deployment is locked by yourself! '
                                              'Do you want to continue?')):
                        abort(colors.red('Deployment is locked!'))
                else:
                    abort(colors.red('Deployment is locked by %s!' % locker))

            self.run('echo %s > .deploying' % self.lock_value())

    def unlock(self, instance):
        with cd(self.project_paths[instance]):
            self.run('rm .deploying')

    def syncdb(self, instance):
        from django.conf import settings
        if django.VERSION >= (1, 7):
            self.run_management_command(instance, 'migrate --noinput')
        elif 'south' in settings.INSTALLED_APPS:
            self.run_management_command(instance, 'syncdb --noinput --migrate')
        else:
            self.run_management_command(instance, 'syncdb --noinput')

        self.notify(colors.green('Synced/Migrated the database'))

    def run_server_updates(self, instance):
        code_dir = self.project_paths[instance]
        with cd(code_dir):
            self.run('git fetch')
            self.run('git reset --hard origin/master')
            self.notify(colors.green('HEAD is now at %s' % self.get_head_hash()[:6]))

            self.run('%s && pip install -r%s' % (self.virtualenv_activate,
                                                 self.requirements[instance]))
            self.notify(colors.green('Updated requirements'))

            self.syncdb(instance)

            from django.conf import settings

            if 'djangobower' in settings.INSTALLED_APPS:
                self.run_management_command(instance, 'bower_install')
                self.notify(colors.green('Ran bower install'))

            if STATIC_FILES in settings.INSTALLED_APPS and settings.STATIC_ROOT is not None:
                self.run_management_command(instance, 'collectstatic --noinput')
                self.notify(colors.green('Collected static files'))

    def restart_app(self, instance):
        if self.restart_command is None:
            # If restart command is not used this method should have been
            # overrided
            raise NotImplementedError
        else:
            self.run(self.restart_command[instance])

    def check_status(self, instance):
        if self.urls is not None:
            if instance in self.urls:
                self.notify(colors.yellow('Checking if %s is alive...' % instance))
                response = requests.get(self.urls[instance]).status_code
                self.status_code = response
                if response != 200:
                    self.notify(colors.red('Sound the alarm, %s did noe respond correctly(%s)' % (
                        instance,
                        response
                    )))

                self.notify(colors.green('Relax already, %s returned 200' % instance))
                return response == 200
            else:
                self.notify(colors.yellow('I have no url for %s' % instance))
                return False

    def deploy(self, instance=None, run_tests=True):
        if instance is None:
            abort(colors.red('You need to provide instance on the form deploy:instance'))
        if bool(run_tests) or confirm('Do you want to run tests before deploying?'):
            self.test(is_deploying=True)

        self.lock(instance)
        if hasattr(self, 'pre_deploy_notify'):
            self.pre_deploy_notify(instance)
        self.run_server_updates(instance)
        self.restart_app(instance)
        self.check_status(instance)
        self.unlock(instance)
        if hasattr(self, 'post_deploy_notify'):
            self.post_deploy_notify(instance)

    def translate(self):
        self.local_management_command('makemessages --all')
        self.notify(colors.green('Made translation files'))

        if confirm('Compile messages?'):
            self.local_management_command('compilemessages')
            self.notify(colors.green('Compiled translation files'))

    def clone_data(self, instance):
        if not confirm('All local data will be replaced with '
                       'data from %s, OK?' % instance):
            abort('Ok!')

        dump_file = '%s.json' % str(int(time.time()))

        # Ignore errors on these next steps, so that we are sure we clean up
        # no matter what
        with settings(warn_only=True) and cd(self.project_paths[instance]):
            # Dump the database to a file...
            self.run_management_command(
                instance,
                '%s --all > %s' % (self.dumpdata_command, dump_file)
            )

            # The download that file, all uploaded files and rm the dump file
            get('%s%s' % (self.project_paths[instance], dump_file), dump_file)
            self.run('rm %s' % dump_file)

            self.syncdb('local')
            self.local_management_command('flush --noinput')

            from django.db import connection, transaction

            cursor = connection.cursor()
            cursor.execute('DELETE FROM django_content_type;')

            for table in self.local_tables_to_flush:
                cursor.execute('DELETE FROM %s;' % table)

            transaction.commit_unless_managed()
            self.local_management_command('%s %s' % (self.loaddata_command, dump_file))

            self.notify(colors.green('Cloned data from %s into the local database' % instance))

        # ... then cleanup the dump file
        self.local('rm %s' % dump_file)



# -*- coding: utf-8 -*-
from django_fabric.base import App


class UwsgiApp(App):
    ini_files = {}

    def __init__(self, ini_files=None, *args, **kwargs):
        super(UwsgiApp, self).__init__(*args, **kwargs)
        self.ini_files = ini_files or self.ini_files

    def restart_app(self, instance):
        self.run('sudo touch %s' % self.ini_files[instance])



import unittest
import os
import django
from django.test.utils import override_settings

from .helpers import TestApp


class TestBaseClass(unittest.TestCase):
    def setUp(self):
        self.log_file_name = 'testlogfile'
        self.fab = TestApp(self.log_file_name)

    def tearDown(self):
        os.remove(self.log_file_name)

    def assertOutput(self, expected_filename):
        if django.VERSION >= (1, 7):
            expected_filename += '1.7'

        with open(self.log_file_name, 'r') as output:
            with open(expected_filename, 'r') as expected:
                self.assertEqual(output.readlines(), expected.readlines())

    def test_deploy(self):
        self.fab.deploy('prod')
        self.assertOutput('tests/expected/deploy')

    @override_settings(STATIC_ROOT=None)
    def test_empty_static_root(self):
        self.fab.deploy('prod')
        self.assertOutput('tests/expected/no_static_root')



# -*- coding: utf8 -*-
import re
import json
import time
import socket
import requests
from fabric import colors


class Notifier(object):
    NICK = 'django-fabric'

    def message_payload(self):
        return {}

    def notification_message_context(self, instance):
        return {
            'url': self.urls[instance],
            'status': self.status_code
        }

    def pre_deploy_notify(self, instance):
        self.send_notification('Deploying %(url)s' % self.notification_message_context(instance))

    def post_deploy_notify(self, instance):
        self.send_notification('Finished deploying %(url)s it responds with status %(status)s' %
                               self.notification_message_context(instance))

    def send_notification(self, message):
        raise NotImplemented


class IrcNotifyMixin(Notifier):
    SERVER = 'irc.freenode.org'
    PORT = 6667
    ROOMS = []
    TIMEOUT = 25

    def post_deploy_notify(self, instance):
        pass

    def send_notification(self, message):
        irc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        irc.settimeout(self.TIMEOUT)
        irc.connect((self.SERVER, self.PORT))
        irc.send("USER %s %s %s :django-fabric\n" % ((self.NICK,) * 3))
        irc.send("NICK %s\n" % self.NICK)
        start = time.time()
        while (time.time() - start) < self.TIMEOUT:
            irc_message = irc.recv(2048).strip('\n\r')
            pong = re.compile(r'^PING\s*:\s*(.*)$').findall(irc_message)

            if pong:
                irc.send("PONG %s\n" % pong)

            if re.findall(' 00[1-4] %s' % self.NICK, irc_message):
                for room in self.ROOMS:
                    irc.send("JOIN %s\n" % room)
                    irc.send("PRIVMSG %s :%s\n" % (room, message))
                    irc.send("PART %s\n" % room)

                break

        irc.send("QUIT\n")
        irc.close()


class SlackNotifyMixin(Notifier):
    CHANNEL = '#general'

    def send_notification(self, message):
        payload = {
            'channel': self.CHANNEL,
            'username': self.NICK,
            'text': message
        }
        data = {'payload': json.dumps(payload)}
        if requests.post(self.URL, data=data).status_code != requests.codes.ok:
            print(colors.yellow('Could not notify Slack'))


class HipChatNotifyMixin(Notifier):
    URL = 'https://api.hipchat.com/v2/room/%s/notification?auth_token=%s'
    COLOR = 'yellow'
    NOTIFY = False

    def send_notification(self, message):
        payload = {
            'message': message,
            'color': self.COLOR,
            'notify': self.NOTIFY
        }
        url = self.URL % (self.ROOM, self.HIPCHAT_TOKEN)
        data = {'payload': json.dumps(payload)}

        if requests.post(url, data=data).status_code != requests.codes.ok:
            print(colors.yellow('Could not notify HipChat'))



from setuptools import find_packages, setup
with open('docs/quickstart.rst', 'r') as f:
    QUICKSTART = f.read()


setup(
    name='django-fabric',
    version='2.0.0',
    author='Rolf Erik Lekang',
    author_email='rolf@mocco.no',
    url='https://github.com/mocco/django-fabric',
    packages=find_packages(),
    description='a generic fabric utility class for django projects',
    long_description=QUICKSTART,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Natural Language :: English",
    ],
    include_package_data=True,
    test_suite='tests.run',
    zip_safe=False,
    install_requires=['fabric', 'django', 'requests'],
    test_requires=['fabric', 'django', 'requests'],
    entry_points={
        'console_scripts': [
            'django-fabric-init = django_fabric.commands:init',
        ]
    }
)



# -*- coding: utf-8 -*-
from fabric.contrib.console import confirm


TEMPLATE = """
from fabric.decorators import task
from fabric.state import env
from django_fabric import App

env.user = '%(server_user)s'
env.hosts = ['%(server)s']

site = App(
    project_paths={
        %(project_paths)s
    },
    %(urls)s
    restart_command={
        %(restart_commands)s
    },
    project_package='%(project_package)s',
    %(test_settings)s
)

deploy = task(site.deploy)
test = task(site.test)
"""


def render_template(settings):
    return TEMPLATE % settings


def init():
    settings = {
        'project_package': raw_input('What is your project package called? '),
        'server_user': raw_input('What is the server user? '),
        'server': raw_input('What is the server host? '),
        'test_settings': '',
        'urls': '',
        '_project_paths': {},
        '_restart_commands': {},
        '_urls': {}
    }

    if confirm('Do you have a test configuration file?', default=False):
        test_settings = raw_input('What is the python path to your test '
                                  'settings(my_app.settings.test)? ')
        settings['test_settings'] = "test_settings='%s'" % test_settings

    instances = raw_input('What is the name of your instances'
                          '(use a commaseperated list)? ').split(',')

    for instance in instances:
        instance = instance.strip()
        settings['_project_paths'][instance] = raw_input(
            'What is the full path of %s instance on the server? ' % instance
        )

        settings['_restart_commands'][instance] = raw_input(
            'What is the restart command for %s '
            'instance on the server? ' % instance
        )

    if confirm('Do you want to check if the site is up after deployment?'):
        for instance in instances:
            instance = instance.strip()
            settings['_urls'][instance] = raw_input(
                'What is the full url of %s instance? ' % instance
            )

        urls = ',\n    '.join(
            ["\'%s\': \'%s\'" % (key, path)
             for key, path in settings['_urls'].items()]
        )

        settings['urls'] = "urls={\n        %s\n    }," % urls

    settings['project_paths'] = ',\n    '.join(
        ["\'%s\': \'%s\'" % (key, path)
         for key, path in settings['_project_paths'].items()]
    )

    settings['restart_commands'] = ',\n    '.join(
        ["\'%s\': \'%s\'" % (key, cmd)
         for key, cmd in settings['_restart_commands'].items()]
    )

    with open('fabfile.py', 'w') as f:
        f.write(render_template(settings))

if __name__ == '__main__':
    init()



from fabric.contrib import django

from django_fabric import App
from django_fabric.test_helpers import TestMixin


class TestApp(TestMixin, App):
    project_package = 'package'
    project_paths = {
        'prod': 'path-to-prod'
    }
    restart_command = {
        'prod': 'restart prod'
    }

    def __init__(self, test_log, *args, **kwargs):
        django.settings_module('tests.django_settings')
        self.test_log = test_log

    def notify(self, message):
        with open(self.test_log, 'a') as log_file:
            log_file.write(message + '\n')

    def lock_value(self):
        """
        Override this to make the expected fixtures work for several people
        """
        return ''
